@@ -8,7 +8,7 @@ package Metrics::Any::Adapter::Test;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 NAME
 
@@ -60,7 +60,8 @@ Each is reported on a line formatted as
    name = value
 
 Each line, including the final one, is terminated by a linefeed. The metrics
-are sorted alphabetically.
+are sorted alphabetically. Any multi-part metric names will be joined with
+underscores (C<_>).
 
 Metrics that have additional labels are formatted with additional label names
 and label values in declared order after the name and before the C<=> symbol:
@@ -125,10 +126,12 @@ sub _make
    my $self = shift;
    my ( $type, $handle, %args ) = @_;
 
+   my $name = $args{name};
+   $name = join "_", @$name if ref $name eq "ARRAY";
+
    $self->{$handle} = {
       type   => $type,
-      # Ugh I really want hash slices but they're 5.20...
-      name   => $args{name},
+      name   => $name,
       labels => $args{labels},
    };
 }
